; Kalender 2006-07-17 - erstellt mit PureBasic 4.00
; Das Programm berechnet fr die Jahre 1 - 3199 sowohl julianische als auch gregorianische Kalender.
; Die Kalender werden in jeweils 2 verschieden Anordnungen der Wochentage erstellt.
; Darberhinaus werden die wesentlichen Jahreskennzeichen ausgegeben.
; Das ganze wird als Textfile im Arbeitsverzeichnis abgelegt und mit Wordpad angezeigt.

Procedure.s roemziffer(wert)
; wandelt wert (1..29) in roemische Zahlen
rz$ = ""
wert1 = wert / 10
wert2 = wert % 10
wert3 = wert % 5
If wert1 > 1
  rz$ + "X"
EndIf
If wert1 > 0
  rz$ + "X"
EndIf
If wert3 = 4
  rz$ + "I"
EndIf
If wert2 > 3 And wert2 < 9
  rz$ + "V"
EndIf
If wert2 = 9
  rz$ + "X"
EndIf
If wert3 < 4
  For i = 1 To wert3
    rz$ + "I"
  Next i
EndIf
ProcedureReturn rz$
EndProcedure

Dim kalender.w(12,6,7)
Dim tage_pro_monat.w(12)
Global Dim tag_l.w(12)
tag_l.w(0) = 0

DataSection
Data.s "Januar","Februar","Mrz","April","Mai","Juni","Juli","August","September","Oktober","November","Dezember"
Data.s "Montag","Dienstag","Mittwoch","Donnerstag","Freitag","Samstag","Sonntag"
Data.s " 000 0   00  000 0 000  00   0 000 ","  1   11  1 1    1    1    1  11111"," 222 2   2    2   2   2   2   22222"
Data.s " 333 3   3    3  33     33   3 333 ","   4   44  4 4 4  4 44444   4    4 ","555555    5     555     55   5 555 "
Data.s " 666 6   66    6666 6   66   6 666 ","77777    7   7   7   7   7    7    "," 888 8   88   8 888 8   88   8 888 "
Data.s " 999 9   99   9 9999    9    9 999 "
Data.s "Rosenmontag","Aschermittwoch","Palmsonntag","Karfreitag","Ostern","Ostermontag","Christi Himmelfahrt","Pfingsten"
Data.s "Pfingstmontag","Fronleichnam","Volkstrauertag","Buss- und Bettag","Totensonntag","1.Sonntag im Advent"
Data.s "2.Sonntag im Advent","3.Sonntag im Advent","4.Sonntag im Advent","1. Weihnachtstag","2. Weihnachtstag","Silvester"
Data.w -48,-46,-7,-2,0,1,39,49,50,60,-42,-39,-35,-28,-21,-14,-7,-6,-5,0
EndDataSection

Global Dim monat_name.s(12)
For i = 1 To 12
  Read monat_name.s(i)
  monat_name.s(i) = LSet(monat_name.s(i),9)
Next i

Dim tag_name.s(7)
For i = 1 To 7
  Read tag_name.s(i)
  tag_name.s(i) = LSet(tag_name.s(i),10)
Next i

Dim bigziffer.s(9)
For i = 0 To 9
  Read bigziffer.s(i)
Next i

Dim feiertage.s(20)
For i = 1 To 20
  Read feiertage.s(i)
  feiertage.s(i) = LSet(feiertage.s(i),20)
Next i

Dim ftag_diff.w(20)
For i = 1 To 20
  Read ftag_diff.w(i)
Next i

; wandelt Tag im Jahr in Datum um
Procedure.s Tag2Datum(tag)
For j = 1 To 12
  If tag <= tag_l.w(j)
    tag$ = RSet(Str(tag - tag_l.w(j-1)),2)
    monat = j
    datum$ = tag$ + "." + monat_name.s(monat)
    ProcedureReturn datum$
  EndIf
Next j
EndProcedure

; bestimmt die Sonntagsbuchstaben
Procedure.s sb(wt, sj)
sb = 7 - wt + 64
If sb > 65
  sb + 32
EndIf
sb$ = Chr(sb)
If sj = 1
  sb = 7 - ((wt + 1) % 7) + 64
  If sb > 65
    sb + 32
  EndIf
  sb$ = sb$ + Chr(sb)
EndIf
ProcedureReturn sb$
EndProcedure

Repeat
  jahr$ = InputRequester("Kalender > file: 'Kalenderxxxx.txt' (C) Eduard Pes","Bitte Jahreszahl 1 bis 3199 eingeben + <ENTER>","")
  jahr = Val(jahr$)
Until jahr > 0 And jahr < 3200

; Berechnungen ber das Jahr 3199 hinaus sind nicht sinnvoll, denn das durch den gregorianischen Kalender geregelte Jahr
; ist um einige Minuten zu lang. Irgendwann muss der Schalttag in durch 400 teilbaren Jahren wieder einmal ausfallen,
; doch konnten sich die Himmelskundigen bisher nicht darber einigen.
; Der 1795 zum Direktor der Pariser Sternwarte ernannte Joseph Lalande pldierte fr das Jahr 3600, der 1877 in Mnster
; verstorbene Astronom Eduard Heis bereits fr 3200 n. Chr.

; Julianisches Jahr
; Die Zhlung der Tage ist innerhalb eines Zeitraums von 7.980 Jahren, der so genannten Julianischen Periode, fortlaufend.
; Die Lnge ergibt sich als kleinstes gemeinsames Vielfaches der Zyklen der Indiktion (15 Jahre), der Goldenen Zahl (19 Jahre)
; und dem Sonnenzyklus (28 Jahre, ergibt sich aus dem 4-jhrigen Schaltjahrzyklus multipliziert mit den 7 Wochentagen).
; Das letzte Zusammenfallen der drei Zyklen (also das letzte mal, das alle drei Zyklen zugleich die Jahresnummer "1" hatten)
; war im Jahr 4713 v. Chr.

jjahr = jahr + 4713

; Schaltjahr
; im julianischen Kalender alle durch 4 teilbaren Jahre im gregrianischen Kalender nicht die durch 100 teilbaren
; jedoch wieder die durch 400 teilbaren Jahre. ; In Schaltjahren wird der Februar um einen so genannten Schalttag,
; den 29. Februar verlngert. ; Dieser ist dann der 60. Tag des Gregorianischen Kalenders.
; Eigentlich wurde jedoch, wie im Julianischen Kalender nach dem 23. Februar ein zweiter 24. Februar eingeschoben.
; Diese Tatsache war noch bis zur Neuordnung des kirchlichen Festkalenders 1969 daran erkenntlich, dass das Fest des
; Hl. Matthias, welches stets am 24. Februar gefeiert wurde, im Schaltjahr auf den 25. Februar verschoben wurde.
; Schalttag
; Dieser war seit rmischer Zeit bis zum Jahr 1970 der 24. Februar. Mit dem Einschieben dieses Tages wurde der 24. und 25. Februar
; zu einem Tag mit 48 Stunden zusammengefasst und erhielten damit auch denselben Tagesbuchstaben.
; Seit 1970 wird der 29. Februar eingeschoben und der Feiertag des Apostel Matthias auch nicht mehr auf den 25. Februar verschoben,
; wie es bis dahin blich war.

If jahr % 4 = 0
  schaltjahr_j = 1
Else
  schaltjahr_j = 0
EndIf

schaltjahr_g = schaltjahr_j

If jahr % 100 = 0
  schaltjahr_g = 0
EndIf
If jahr % 400 = 0
  schaltjahr_g = 1
EndIf

; Gregorianische Korrektur
; sie ist gleich der im gregorianischen Kalender gegenber dem julianischen ausgefallenen Schalttage seit dem Jahr 201

korrektur_g = jahr / 100 - jahr / 400 - 2

; Julianischer Tag am 0.Januar des Jahres

jultag0_j = ((jjahr + 2) * 36525) / 100 - 1096
jultag0_g = jultag0_j - (jahr-1) / 100 + (jahr-1) / 400 + 2

; Goldene Zahl
; Zyklische Mondberechnung
; Der Berechnung der Vollmonde legte man den sogenannten Metonischen Zyklus zugrunde. Der Grieche Meton hatte, wie vor ihm schon
; babylonische Astronomen, erkannt, da 235 Mondmonate ziemlich genau 19 Jahren entsprachen. Unter dieser Voraussetzung muten
; die Neumonde nach 19 julianischen Jahren wieder auf dieselben Daten fallen. Zur Bezeichnung der Jahre innerhalb dieses 19jhrigen
; Zyklus' errechnete man die sogenannte Goldene Zahl. Sie ist der um eins vermehrte Rest der Division der Jahreszahl durch 19.
; Zum Beispiel hatte das Jahr 1492 die Goldene Zahl 11.
; In allen Jahren mit der gleichen Goldenen Zahl muten nun die Neumonde auf die gleichen Daten fallen.
; Es wurde festgestellt, da in den Jahren mit der Goldenen Zahl 19 ein Neumond auf den 24. Dezember fiel. Da das kirchliche Jahr
; mit dem 25. Dezember, dem Weihnachtstage, begann, setzte man den Beginn des 19jhrigen Zyklus auf den 24. Dezember.
; Die Datierung der Tage in diesem Mondkalender beruhte auf Monaten von abwechselnd 30 und 29 Tagen, beginnend mit einem 
; 30tgigen Monat. Der 24. Dezember war der erste Tag des ersten 30tgigen Mondmonats, und dessen letzter Tag war somit der
; 22. Januar des neuen Jahres. Mit dem 23. Januar begann ein 29tgiger Mondmonat usf.
; Da ein Mondmonat etwas lnger als 29,5 Tage ist, wurden im Laufe der 19 Jahre des Mondzyklus siebenmal Schaltmonate mit
; 30 Tagen eingefgt. Die Schalttage der Schaltjahre schob man ein, ohne an ihnen den Mondkalender weiterzuzhlen. 
; Um den neuen Mondzyklus wieder mit dem 24. Dezember beginnen zu lassen, mute im letzten Jahr des Zyklus ein Tag fortgelassen werden.
; Diese Weglassung nannte man den saltus lunae.
; Durch diese Anordnung hatte ein Mondmonat eine durchschnittliche Lnge von 29,53085 Tagen. Die wahre Lnge eines (synodischen) Monats
; ist jedoch 29,53059 Tage. Der Unterschied zwischen zyklischem und wahrem Neumond wuchs in etwa 310 Jahren zu einem Tag an.
; die Goldene Zahl wird traditionell in rmischen Ziffern angegeben.

goldene_zahl = ((jjahr - 1) % 19) + 1

; Indiktion
; Im Mittelalter war es blich bei Datumsangaben die Rmerzinszahl, Indiktionszyklus genannt, anzugeben. Es handelt sich dabei
; um einen im Rmischen Reich eingefhrten fnfzehnjhrigen Grundsteuerzyklus. (indictio = kaiserliche Verfgung).

indiktion = ((jjahr - 1) % 15) + 1

; Sonnenzirkel
; Die Verteilung der Wochentage auf das Kalenderjahr hat einen Zyklus oder Zirkel von 7 x 4 = 28 Jahren.
; 7 von den Wochentagen und 4 von den Schaltjahren. Sonnenzirkel ist die Nummer des Jahres im Zyklus

sonnenzirkel = ((jjahr - 1) % 28) + 1

; Epakten neuen Stils
; Die gregorianische Reform des julianischen Kalenders geht auf die Gelehrten Aloysius Lilius, eigentlich Liugi Lilio
; (auch Giglio) Ghiraldi, und Christoph Clavius, eigentlich Christoph Klau, zurck.
; Der der verbesserten Osterrechnung zugrunde gelegte Mondzyklus hatte ebenfalls eine Lnge von 19 Jahren, wurde jedoch zu bestimmten
; Zeitpunkten korrigiert.
; Die Epakten neuen Stils, die jetzt das Mondalter des 1. Januar bezeichneten, werden als eine Schpfung Lilius' angesehen.
; Auch in der neuen Mondberechnung wechselten 30- und 29tgige Mondmonate ab. Die Neumonde wurden durch die Epakte derjenigen
; Jahre gekennzeichnet, in denen sie auftraten.
; Zum Zeitpunkt der Einfhrung des Gregorianischen Kalenders trugen die Jahre mit der Goldenen Zahl 1 die Epakte 1, d. h. auf den
; 31. Dezember des Vorjahres fiel ein (zyklischer) Neumond. Daher trat neben den 30. Januar eine 1, weil der erste Neumond
; in diesem Jahr an diesem Tage eintrat. Alle folgenden in Jahren mit der Goldenen Zahl 1 auftretenden Neumonde wurden ebenfalls
; mit einer 1 gekennzeichnet. Die Jahresepakte der Jahre mit der Goldenen Zahl 2 betrug 12, so da alle Neumondtage dieses Jahres
; mit einer 12 gekennzeichnet wurden. Ebenso verfuhr man mit den weiteren Jahren. Die verbleibenden Lcken fllte man abwrtszhlend
; auf, wobei in den 29-tgigen Monaten die Zahlen 25 und 24 an einem Tag nebeneinander geschrieben wurden.
; Auf diese Weise erhielt man den sog. Immerwhrenden Gregorianischen Kalender.
epakte_j = ((goldene_zahl - 3) * 11) % 30
While epakte_j < 0
  epakte_j + 30
Wend

; Sonnenangleichung
; jeder ausfallende Schalttag setzt die Epakte um 1 herunter entspricht also dem negativen Wert der gregorianischen Korrektur
sonnenangleichung = - korrektur_g

; Mondangleichung
; Regel: siebenmal alle 300 Jahre und das achte mal nach 400 Jahren einen Tag vorrcken. Der Beginn wurde auf das Jahr 1800
; festgelegt
; da hier nur bis zum Jahr 3199 grechnet wird, reicht die folgende Berechnung - sie ist bis zum Jahr 4199 korrekt
mondangleichung = (jahr / 100 - 15) / 3
epakte_g = (((goldene_zahl) * 11) + sonnenangleichung + mondangleichung) % 30
While epakte_g < 0
  epakte_g + 30
Wend

; Sonntagsbuchstabe
; Bezeichnet man die Tage des Jahres zyklisch mit den Buchstaben von A bis G, so erhlt man einen immerwhrenden
; Kalender, bei dem nur die Zuordnung der Wochentage zu den Buchstaben wechselt.
; Der Wochentag wird dadurch festgelegt, da man fr jedes Jahr jenen Buchstaben anfhrt, der dem ersten Sonntag des
; Jahres zugeordnet ist; das ist der Sonntagsbuchstabe.
; In Schaltjahren erhlt der Schalttag den gleichen Sonntagsbuchstaben wie der vorhergehende Tag. Es gibt also
; 2 Sonntagsbuchstaben - einen bis zum Schalttag und einen
; fr die restlichen Tage.
; 0..6 Mo..So
wochentag_j = (jultag0_j + 1) % 7
wochentag_g = (jultag0_g + 1) % 7

; Sonntagsbuchstabe
sonntagsbuchstabe_j$ = sb(wochentag_j, schaltjahr_j)
sonntagsbuchstabe_g$ = sb(wochentag_g, schaltjahr_g)

; Ostergrenze
ostergrenze_j = 90 + (30 - epakte_j) + 13
If ostergrenze_j > 109
  ostergrenze_j - 30
EndIf

ostergrenze_g = 90 + (30 - epakte_g) + 13
If ostergrenze_g > 109
  ostergrenze_g - 30
EndIf

; Ausnahme 1
; die spteste Ostergrenze, der 19.April, wird regelmig in den 18. April umgendert
If ostergrenze_g = 109
  ostergrenze_g - 1
EndIf

; Ausnahme 2
; die davor liegende Ostergrenze, der 18.April, wird in den 17.April umgendert, wenn die Goldene Zahl > 11 ist
If ostergrenze_g = 108 And goldene_zahl > 11
  ostergrenze_g - 1
EndIf

wochentagog_j = (jultag0_j + ostergrenze_j + schaltjahr_j) % 7
wochentagog_g = (jultag0_g + ostergrenze_g + schaltjahr_g) % 7

; Ostern
; Die Osterberechnungen nach julianischem bzw. gregorianischem Modus bercksichtigen folgende Abweichungen nicht:
; 1. Die Festsetzung des Ostertermines in Teilen der sptantiken/frhmittelalterlichen Kirche nach den
;    verschiedenen alternativen Osterzyklen vor und neben dem des Dionysius exiguus.
; 2. Abweichende Ansetzungen wegen anderer Grenztage (Teil des Osterstreits der alten Kirche).
; 3. Abweichende Ostertermine in protestantischen Lndern, die 
;    a) den gregorianischen Kalender zunchst nur als als brgerlichen annahmen.
;    b) am julianischen Kalender festhielten, aber dennoch Korrekturen am Schaltzyklus vornahmen.
dif = 6 - wochentagog_j
If dif < 1
  dif + 7
EndIf
ostern_j = ostergrenze_j + dif

dif = 6 - wochentagog_g
If dif < 1
  dif + 7
EndIf
ostern_g = ostergrenze_g + dif

; Festzahl
festzahl_j = ostern_j - 80
festzahl_g = ostern_g - 80

; Kalender-file erzeugen
If CreateFile(1, "Kalender" + jahr$ +".txt")
  M = MessageRequester("Kalender","Der Kalender befindet sich als Textfile: 'Kalender"+jahr$+".txt' im Verzeichnis diese Programms", #PB_MessageRequester_Ok )
Else
  Ergebnis = MessageRequester("Kalender","konnte file: 'Kalenderxxxx.txt' nicht erstellen", #PB_MessageRequester_Ok )
  End
EndIf

Dim stelle.w(4)
stelle(1) = jahr / 1000
stelle(2) = (jahr / 100) % 10
stelle(3) = (jahr / 10) % 10
stelle(4) = jahr % 10

wochentag  = wochentag_g
schaltjahr = schaltjahr_g
ostern     = ostern_g
k$         = "gregorianisch"
Gosub PrintKalender

wochentag  = wochentag_j
schaltjahr = schaltjahr_j
ostern     = ostern_j
k$         = "julianisch"
Gosub PrintKalender

WriteStringN(1,"Jahreskennzeichen - gregorianisch / julianisch")
WriteStringN(1,LSet("Julianisches Jahr",26) + Str(jjahr) + " / " + Str(jjahr))
WriteStringN(1,LSet("Schaltjahr",26) + Str(schaltjahr_g) + " / " + Str(schaltjahr_j))
WriteStringN(1,LSet("gregorianische Korrektur",26) + Str(korrektur_g))
WriteStringN(1,LSet("Jul. Tag am 0.Januar",26) + Str(jultag0_g) + " / " + Str(jultag0_j))
WriteStringN(1,LSet("Sonnenzirkel",26) + Str(sonnenzirkel) + " / " + Str(sonnenzirkel))
WriteStringN(1,LSet("Mondzirkel = Goldene Zahl",26) + roemziffer(goldene_zahl) + " / " + roemziffer(goldene_zahl))
WriteStringN(1,LSet("Indiktion",26) + Str(indiktion) + " / " + Str(indiktion))
WriteStringN(1,LSet("Sonnenangleichung",26) + Str(sonnenangleichung))
WriteStringN(1,LSet("Mondangleichung",26) + Str(mondangleichung))
WriteStringN(1,LSet("Epakte",26) + Str(epakte_g) + " / " + Str(epakte_j))
WriteStringN(1,LSet("Wochentag am 1. Januar",26) + RTrim(tag_name.s(wochentag_g+1)) + " / " + RTrim(tag_name.s(wochentag_j+1)))
WriteStringN(1,LSet("Sonntagsbuchstabe(n)",26) + sonntagsbuchstabe_g$ + " / " + sonntagsbuchstabe_j$)
WriteStringN(1,LSet("Ostergrenze",26) + Left(tag_name.s(wochentagog_g+1), 2) + "," + RTrim(Tag2Datum(ostergrenze_g + schaltjahr_g)) + " / " + Left(tag_name.s(wochentagog_j+1), 2) + "," + RTrim(Tag2Datum(ostergrenze_j + schaltjahr_j)))
WriteStringN(1,LSet("Festzahl",26) + Str(festzahl_g) + " / " + Str(festzahl_j))
WriteStringN(1,"")
CloseFile(1)
RunProgram("Wordpad", "Kalender"+jahr$+".txt" , "" ) 
End

PrintKalender:
; Anzahl Tage der Monate
p = 0
For i = 1 To 12
  If i > 1
    n = (306 * (i + 2)) / 10 - 63 + schaltjahr
  Else
    n = (306 * (i + 14)) / 10 - 428
  EndIf
  tage_pro_monat.w(i) = n - p
  tag_l.w(i) = n
  p = n
Next i

; Die Kalendertage in das Array ordnen
m = (wochentag - 1) * -1
If m > -2
  m = m - 7
EndIf

For i = 1 To 12
  For j = 1 To 6
    For k = 1 To 7
      kalender.w(i,j,k) = 0
    Next k
  Next j
Next i

For i = 1 To 12
  If i > 2
    l = m - (306 * (i + 1) / 10 - 63 + schaltjahr) % 7
  Else
    l = m - (306 * (i + 13) /10 - 428) % 7
  EndIf
  While l < -5
    l + 7
  Wend
  For j = 1 To 6
    For k = 1 To 7
      If l > 0 And l <= tage_pro_monat(i)
        kalender.w(i,j,k) = l
      EndIf
      l + 1
    Next k
  Next j
Next i

sp = 24
Gosub PrintBig
i = 1
Repeat
  WriteStringN(1,"")
  WriteString(1,Space(6))
  For j = 1 To 3
    WriteString(1,Space(12) + monat_name.s(i+j-1))
  Next j
   WriteStringN(1,"")
    WriteStringN(1,"")
  For k = 1 To 7
    WriteString(1,tag_name(k))
    For l = i To i+2
      WriteString(1,"   ")
      For m = 1 To 6
        If kalender.w(l,m,k) = 0
          WriteString(1,"   ")
        Else
          WriteString(1,RSet(Str(kalender.w(l,m,k)),3))
        EndIf
      Next m
    Next l
    WriteStringN(1,"")
  Next k
  WriteStringN(1,"")
  i + 3
Until i => 12
sp = 3
Gosub PrintFeiertage

sp = 20
Gosub PrintBig
i = 1
Repeat
  WriteStringN(1,"")
  For j = 1 To 3
    WriteString(1,"  " + monat_name.s(i+j-1) + Space(13))
  Next j
  WriteStringN(1,"")
  WriteStringN(1,"")
  For j = 1 To 3
    For k = 1 To 7
      WriteString(1,Left(tag_name(k), 2) + " ")
    Next k
    WriteString(1,"   ")
  Next j
  WriteStringN(1,"")
  For m = 1 To 6
    For l = i To i+2
      For k = 1 To 7
        If kalender.w(l,m,k) = 0
          WriteString(1,"   ")
        Else
          WriteString(1,RSet(Str(kalender.w(l,m,k)),2) + " ")
        EndIf
      Next k
      WriteString(1,"   ")
    Next l
    WriteStringN(1,"")
  Next m
  WriteStringN(1,"")
  i + 3
Until i => 12
sp = 1
Gosub PrintFeiertage
Return

PrintBig:
; erzeugt grosse Ziffern in 5x7 Matrix
For i = 0 To 6
  WriteString(1,Space(sp))
  For j = 1 To 4
    WriteString(1,"   "+Mid(bigziffer.s(stelle(j)),5*i+1,5))
  Next j
  WriteStringN(1,"")
Next i
WriteStringN(1,k$)
Return

PrintFeiertage:
For i = 1 To 10
  ftag = ostern + schaltjahr + ftag_diff(i)
  WriteString(1,feiertage.s(i) + ": " + Tag2Datum(ftag))
  ftag = tag_l.w(12) + ftag_diff(i+10)
  If i < 8
    ftag = ftag - (wochentag + 1 + schaltjahr) % 7
  EndIf
  If jahr < 1952 And i = 1 ; Totensonntag erst ab 1952
    WriteStringN(1,"")
  Else
    WriteStringN(1,Space(sp) + feiertage.s(i + 10) + ": " + Tag2Datum(ftag))
  EndIf
Next i
WriteStringN(1,Chr(12))
Return


; IDE Options = PureBasic v4.00 (Windows - x86)
; CursorPosition = 340
; FirstLine = 318
; Folding = -
; UseIcon = D:\PureBasic391\EigeneProgramme\favicon.ico
; Executable = ..\..\PureBasic391\EigeneProgramme\kalender.exe